@@ -5,7 +5,7 @@ import time
 import uuid
 from configparser import ConfigParser
 from contextlib import closing
-from ipaddress import ip_address, ip_interface
+from ipaddress import ip_address, ip_interface, ip_network
 from pathlib import Path
 from shutil import rmtree
 from socket import AF_INET, AF_INET6, IPPROTO_TCP, SOCK_DGRAM, socket
@@ -457,6 +457,27 @@ class NMManager:
         _logger.debug("writing wireguard configuration to Network Manager")
         ipv4s = []
         ipv6s = []
+        lan_4 = []
+        lan_6 = []
+        # set LANs to whitelist
+        # we cannot just add a ip rule that takes everything from main table except the default route
+        # as we have Tunnelcrack/Tunnelvision attacks to worry about
+        # from RFC1918: https://datatracker.ietf.org/doc/html/rfc1918#section-3
+        if allow_wg_lan:
+            lan_4 = [
+                ip_network("10.0.0.0/8"),
+                ip_network("172.16.0.0/12"),
+                ip_network("192.168.0.0/16"),
+            ]
+            # from RFC4193 https://datatracker.ietf.org/doc/html/rfc4193#section-3.1
+            lan_6 = [ip_network("fc00::/7")]
+        # we check if traffic to the VPN ip is in one of the LAN ranges
+        # if so, then we need to ensure that this traffic is going to the WG routing table
+        # We can't just add a routing rule that it is always going to the WG routing table
+        # as there could be public IPs here, and then we would get a routing loop
+        needs_exclude_v4 = []
+        needs_exclude_v6 = []
+
         self.wg_gateway_ip = None
         for ip in config["Interface"]["Address"].split(","):
             addr = ip_interface(ip.strip())
@@ -467,8 +488,19 @@ class NMManager:
                         raise RuntimeError("common lib not available")
                     self.wg_gateway_ip = self.common_lib.calculate_gateway(net_str)
                 ipv4s.append(NM.IPAddress(AF_INET, str(addr.ip), addr.network.prefixlen))
+
+                # if this VPN v4 IP is in one of the LAN v4 ranges
+                # we need to exclude this from the LAN v4 allow list
+                for lan in lan_4:
+                    if addr in lan:
+                        needs_exclude_v4.append((str(addr.ip), addr.network.prefixlen))
             elif addr.version == 6:
                 ipv6s.append(NM.IPAddress(AF_INET6, str(addr.ip), addr.network.prefixlen))
+                # if this VPN v6 IP is in one of the LAN v6 ranges
+                # we need to exclude this from the LAN v6 allow list
+                for lan in lan_6:
+                    if addr in lan:
+                        needs_exclude_v6.append((str(addr.ip), addr.network.prefixlen))
 
         dns4 = []
         dns6 = []
@@ -504,6 +536,13 @@ class NMManager:
         wg_endpoint = config["Peer"]["Endpoint"]
         peer.set_endpoint(wg_endpoint, allow_invalid=False)
 
+        psk = config["Peer"].get("PresharedKey")
+        if psk:
+            peer.set_preshared_key(psk, accept_invalid=False)
+            # for some reason this needs to be set
+            # AGENT_OWNED doesn't work
+            peer.set_preshared_key_flags(NM.SettingSecretFlags.NONE)
+
         peer.set_public_key(config["Peer"]["PublicKey"], accept_invalid=False)
         for ip in config["Peer"]["AllowedIPs"].split(","):
             peer.append_allowed_ip(ip.strip(), accept_invalid=False)
@@ -576,17 +615,8 @@ class NMManager:
         # We want to make this configurable
         # Additionally, the overlap case with split tunnel doesn't work: https://codeberg.org/eduvpn/linux-app/issues/551
 
-        # set LANs to whitelist
-        # we cannot just add a ip rule that takes everything from main table except the default route
-        # as we have Tunnelcrack/Tunnelvision attacks to worry about
-        # from RFC1918: https://datatracker.ietf.org/doc/html/rfc1918#section-3
-        lan_4 = [("10.0.0.0", 8), ("172.16.0.0", 12), ("192.168.0.0", 16)]
-        # from RFC4193 https://datatracker.ietf.org/doc/html/rfc4193#section-3.1
-        lan_6 = [("fc00::", 7)]
-        rules = [(4, AF_INET, s_ip4, 32, lan_4), (6, AF_INET6, s_ip6, 128, lan_6)]
-        # priority 1 not fwmark fwmarknum table fwmarknum
-
         prios = self.get_priorities(proxy is not None, allow_wg_lan)
+        rules = [(4, AF_INET, s_ip4, 32, lan_4), (6, AF_INET6, s_ip6, 128, lan_6)]
         for ipver, family, setting, subnet, lans in rules:
             rule = NM.IPRoutingRule.new(family)
             rule.set_priority(prios[0])
@@ -613,17 +643,20 @@ class NMManager:
 
             # when LAN should be allowed, we have to add a higher priority suppress prefixlength rule
             if allow_wg_lan:
-                exclude_rule = NM.IPRoutingRule.new(family)
-                exclude_rule.set_priority(prios[3])
-                exclude_rule.set_invert(True)
-                # fwmask 0xffffffff is the default
-                exclude_rule.set_fwmark(fwmark, 0xFFFFFFFF)
-                exclude_rule.set_table(fwmark)
-                exclude_rule.set_suppress_prefixlength(0)
-                setting.add_routing_rule(exclude_rule)
-                for ipr in lans:
+                # make sure that traffic to the VPN ip range doesn't match the LAN rules
+                exclude_ips = needs_exclude_v6
+                if ipver == 4:
+                    exclude_ips = needs_exclude_v4
+                for ipaddr, subnet in exclude_ips:
+                    exclude_rule = NM.IPRoutingRule.new(family)
+                    exclude_rule.set_priority(prios[3])
+                    exclude_rule.set_to(ipaddr, subnet)
+                    exclude_rule.set_table(fwmark)
+                    exclude_rule.set_suppress_prefixlength(0)
+                    setting.add_routing_rule(exclude_rule)
+                for lan in lans:
                     lan_rule = NM.IPRoutingRule.new(family)
-                    lan_rule.set_to(ipr[0], ipr[1])
+                    lan_rule.set_to(str(lan.network_address), lan.prefixlen)
                     lan_rule.set_priority(prios[2])
                     lan_rule.set_suppress_prefixlength(0)
                     setting.add_routing_rule(lan_rule)
